<?php
/**
 * @package		Remarkety
 * @copyright	Copyright Interamind Ltd © 2014 - All rights reserved.
 * @license		GNU/GPL
 * @author		Remarkety 
 * @author mail	support@remarkety.com
 * @website		www.remarkety.com
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.controller');


define('REMARKETY_STATUS', 'STATUS');
define('REMARKETY_SUCCESS_STATUS', 'SUCCESS');
define('REMARKETY_FAILED_STATUS', 'FAILED');
define('REMARKETY_ERROR', 'ERROR');
define('API_KEY_VALIDATION_FAILED', 'API_KEY_VALIDATION_FAILED');
define('REMARKETY_DATA', 'DATA');
define('REMARKETY_DEBUG', 'DEBUG');
define('REMARKETY_VM_VERSION', 'VM_VERSION');
define('REMARKETY_PLUGIN_VERSION', 'PLUGIN_VERSION');


define('com_remarkety', 'com_remarkety');


require_once JPATH_SITE.DS.'components'.DS.'com_remarkety'.DS.'remarketyLogger.php';

class remarketyController extends JController
{
	var $sendDebugInfo = false;
	var $debugInfo = array();
	var $isInitialzedVM = false;
	var $remarketyModel = null;
	
	/**
	 * Custom Constructor
	 */

	function __construct()
	{
		parent::__construct();
		$this->remarketyModel = $this->getRemarketyModel();
		$this->loadVirtueMartFiles();
		
		$isDeleteExpiredCoupons = self::getParamByName('isDeleteExpiredCoupons', false);
        if(!empty($isDeleteExpiredCoupons)){
        	$this->remarketyModel->deleteExpiredCoupons();
        }
	}
	
	function display() {
        $this->validateApiKey();
		$this->returnJson(null, REMARKETY_SUCCESS_STATUS);
	}
	
	/**
	 * Available URL Query parameters:

    limit — Amount of results (default: 50) (maximum: 250)
    page — Page to show (default: 1)
    since_id — Restrict results to after the specified ID
    created_at_min — Show orders created after date (format: 2008-01-01 03:00)
    created_at_max — Show orders created before date (format: 2008-01-01 03:00)
    updated_at_min — Show orders last updated after date (format: 2008-01-01 03:00)
    updated_at_max — Show orders last updated before date (format: 2008-01-01 03:00)
    status
        pending - All open orders (default)
        confirmed - Show only closed orders
        cancelled - Show only cancelled orders
        any - Any order status
	 */
	function orders(){
    	global $mainframe;
		$this->validateApiKey();
		
		try {
			$model = & $this->getRemarketyModel('');
	        $limit = urldecode(JRequest::getVar( 'limit', '50' ));
	        $page = urldecode(JRequest::getVar( 'page', '0' ));
	        $since_id = urldecode(JRequest::getVar( 'since_id' ));
	        $created_at_min = urldecode(JRequest::getVar( 'created_at_min' ));
	        $created_at_max = urldecode(JRequest::getVar( 'created_at_max' ));
	        $updated_at_min = urldecode(JRequest::getVar( 'updated_at_min' ));
	        $updated_at_max = urldecode(JRequest::getVar( 'updated_at_max' ));
	        $status = urldecode(JRequest::getVar( 'status' ));
	        $order_id = urldecode(JRequest::getVar( 'order_id' ));
	        
	        $orders = $model->getOrders($limit, $page, $since_id, $created_at_min, $created_at_max, $updated_at_min, $updated_at_max, $status, $order_id, $this->debugInfo);
	       	$this->returnJson($orders, REMARKETY_SUCCESS_STATUS);
        
		} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	/**
	 * Receive a list of all Products

	Available URL Query parameters:

    limit — Amount of results (default: 50) (maximum: 250)
    page — Page to show (default: 1)
    since_id — Restrict results to after the specified ID
    vendor — Filter by product vendor
    handle — Filter by product handle
    product_type — Filter by product type
    collection_id — Filter by collection id
    created_at_min — Show products created after date (format: 2008-01-01 03:00)
    created_at_max — Show products created before date (format: 2008-01-01 03:00)
    updated_at_min — Show products last updated after date (format: 2008-01-01 03:00)
    updated_at_max — Show products last updated before date (format: 2008-01-01 03:00)
    published_at_min — Show products published after date (format: 2008-01-01 03:00)
    published_at_max — Show products published before date (format: 2008-01-01 03:00)
    published_status
        published - Show only published products
        unpublished - Show only unpublished products
        any - Show all products (default)
    fields — comma-separated list of fields to include in the response
	 */
	
	public function products(){
		global $mainframe;
		$this->validateApiKey();
		
		try {
			$model = & $this->getRemarketyModel();
    		$limit = urldecode(JRequest::getVar( 'limit', '50' ));
	        $page = urldecode(JRequest::getVar( 'page', '0' ));
	        $since_id = urldecode(JRequest::getVar( 'since_id' ));
	    	$handle = urldecode(JRequest::getVar( 'handle' ));
	    	$vendor = urldecode(JRequest::getVar( 'vendor' ));
	    	$product_type = urldecode(JRequest::getVar( 'product_type' ));
	    	$collection_id = urldecode(JRequest::getVar( 'collection_id' ));
	        $created_at_min = urldecode(JRequest::getVar( 'created_at_min' ));
	        $created_at_max = urldecode(JRequest::getVar( 'created_at_max' ));
	        $updated_at_min = urldecode(JRequest::getVar( 'updated_at_min' ));
	        $updated_at_max = urldecode(JRequest::getVar( 'updated_at_max' ));
	        $published_at_min = urldecode(JRequest::getVar( 'published_at_min' ));
	        $published_at_max = urldecode(JRequest::getVar( 'published_at_max' ));
	        $published_status = urldecode(JRequest::getVar( 'published_status', 'any' ));
	        $product_id = urldecode(JRequest::getVar( 'product_id', null ));
	        
	        $products = $model->getProducts($limit, $page, $since_id, $handle, $vendor, $product_type, $collection_id, 
	        		$created_at_min, $created_at_max, $updated_at_min, $updated_at_max, $published_at_min, $published_at_max, $published_status, $product_id, $this->debugInfo);

	       	$this->returnJson($products, REMARKETY_SUCCESS_STATUS);
    		
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function shoppers(){
		global $mainframe;
		$this->validateApiKey();
		
		try {
			$model = & $this->getRemarketyModel();
    		$limit = urldecode(JRequest::getVar( 'limit', '50' ));
	        $page = urldecode(JRequest::getVar( 'page', '0' ));
	        $since_id = urldecode(JRequest::getVar( 'since_id' ));
	    	$updated_at_min = urldecode(JRequest::getVar( 'updated_at_min' ));
	        
	        $shoppers = $model->getshoppers($limit, $page, $since_id, $updated_at_min, $this->debugInfo);
    		$this->returnJson($shoppers, REMARKETY_SUCCESS_STATUS);
    		
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function carts(){
		global $mainframe;
		$this->validateApiKey();
		
		try {
			$model = & $this->getRemarketyModel();
    		$limit = urldecode(JRequest::getVar( 'limit', '50' ));
	        $page = urldecode(JRequest::getVar( 'page', '0' ));
	        $since_id = urldecode(JRequest::getVar( 'since_id' ));
	    	$updated_at_min = urldecode(JRequest::getVar( 'updated_at_min' ));
	    	$updated_at_max = urldecode(JRequest::getVar( 'updated_at_max' ));
	        
	        $carts = $model->getCarts($limit, $page, $since_id, $updated_at_min, $updated_at_max, $this->debugInfo);
    		$this->returnJson($carts, REMARKETY_SUCCESS_STATUS);
    		
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function createCoupon(){
		global $mainframe;
		$this->validateApiKey();
		
		try {
    		$model = & $this->getRemarketyModel();
    		$code = urldecode(JRequest::getVar( 'code', '' ));
	        $percent_or_total = urldecode(JRequest::getVar( 'percent_or_total', 'percent' ));
	        $gift_or_permanent = urldecode(JRequest::getVar( 'gift_or_permanent', 'gift' ));
	        $value = urldecode(JRequest::getVar( 'value', '' ));
	    	$startDate = urldecode(JRequest::getVar( 'startDate' ));
	    	$expiryDate = urldecode(JRequest::getVar( 'expiryDate' ));
	        $validFromValue = urldecode(JRequest::getVar( 'validFromValue' ));
	        
	        $coupon_result = $model->createCoupon($code, $percent_or_total, $gift_or_permanent, $value, $startDate, $expiryDate, $validFromValue, $this->debugInfo); 
	        
	        remarketyLogger::log('contoller createCoupon coupon_result = '.$coupon_result, remarketyLogger::LEVEL_DEBUG);
    		$this->returnJson($coupon_result, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	/**
    "address1": "1 Infinite Loop",
    "city": "Cupertino",
    "country": "US",
    "created_at": "2007-12-31T19:00:00-05:00",
    "customer_email": "customers@apple.com",
    "domain": "shop.apple.com",
    "email": "steve@apple.com",
    "name": "Apple Computers",
    "phone": null,
    "province": "California",
    "zip": "95014",
    "currency": "USD",
    "timezone": "(GMT-05:00) Eastern Time (US & Canada)",
    "shop_owner": "Steve Jobs",
    "money_format": "$ {{amount}}",
    "money_with_currency_format": "$ {{amount}} USD",
    "taxes_included": null,
    "tax_shipping": null,
	 */
	function shop(){
		global $mainframe;
		$this->validateApiKey();
		
		try {
    		$model = & $this->getRemarketyModel();
    		$shopSettings = $model->getShopSettings($this->debugInfo);
    		$this->returnJson($shopSettings, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function getShoppersCount(){
		global $mainframe;
		$this->validateApiKey();
    	
		try {
			$model = & $this->getRemarketyModel();
    		$shoppersCount = $model->getShoppersCount();
    		$this->returnJson($shoppersCount, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function getOrdersCount(){
		global $mainframe;
		$this->validateApiKey();
    	
		try {
			$model = & $this->getRemarketyModel();
    		$ordersCount = $model->getOrdersCount();
    		$this->returnJson($ordersCount, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function getProductsCount(){
		global $mainframe;
		$this->validateApiKey();
    	
		try {
			$model = & $this->getRemarketyModel();
    		$productsCount = $model->getProductsCount();
    		$this->returnJson($productsCount, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	public function statuses(){
		global $mainframe;
		$this->validateApiKey();
    	
		try {
			$model = & $this->getRemarketyModel();
    		$shopStatuses = $model->getShopStatuses($this->debugInfo);
    		$this->returnJson($shopStatuses, REMARKETY_SUCCESS_STATUS);
    	} catch (Exception $e) {
    		$this->returnJson(null, REMARKETY_FAILED_STATUS, $e->getMessage());
    	}
	}
	
	private function getVmVersion(){
		require_once (JPATH_ADMINISTRATOR.DS."components".DS."com_virtuemart".DS."version.php");
		$VMVERSION = new vmVersion();
		$versionArr['PRODUCT'] = $VMVERSION->PRODUCT;
		$versionArr['RELEASE'] = $VMVERSION->RELEASE;
		$versionArr['DEV_STATUS'] = $VMVERSION->DEV_STATUS;
		$versionArr['CODENAME'] = $VMVERSION->CODENAME;
		$versionArr['RELDATE'] = $VMVERSION->RELDATE;
		$versionArr['RELTIME'] = $VMVERSION->RELTIME;
		$versionArr['RELTZ'] = $VMVERSION->RELTZ;
		$versionArr['REVISION'] = $VMVERSION->REVISION;
		return $versionArr;
	}
	
	private function getPluginVersion(){
		$version = $this->getComponetManifestElement('version');
		return $version;
	}

	private static function getComponetManifestElement($elementName) {
		$manifest = new SimpleXMLElement(file_get_contents(JPATH_ADMINISTRATOR .DS. 'components'.DS.'com_remarkety'.DS.'manifest.xml'));
		$com_version = $manifest->$elementName;
		return (string)$com_version;
	}
	
	private function returnJson($data, $status, $error = null){
		$json[REMARKETY_DATA] = $data;
		$json[REMARKETY_STATUS] = $status;
		$json[REMARKETY_ERROR] = $error;
		$json[REMARKETY_PLUGIN_VERSION] = $this->getPluginVersion();
		
		if ($this->sendDebugInfo) {
			if (!empty($this->debugInfo)) {
				$json[REMARKETY_DEBUG] = $this->debugInfo;
			}
		}
		
		if($error != API_KEY_VALIDATION_FAILED)
			$json[REMARKETY_VM_VERSION] = $this->getVmVersion();
        
        JResponse::setHeader('Content-Type', 'application/json; charset=utf-8', true);
		JResponse::sendHeaders();
        $encoded = json_encode($json);
        echo $encoded;
        $this->close();
	}
	
	private function close(){
		$mainframe =& JFactory::getApplication();
        $mainframe->close();
	}
	
	private function validateApiKey(){
		$request_api_key = urldecode(JRequest::getVar( 'remarkety_api_key', '' ) );
		
		$this->sendDebugInfo = JRequest::getVar('debug', false) != false;
		
		if(empty($request_api_key) || strlen($request_api_key) > 50){
			$this->returnJson(null, REMARKETY_FAILED_STATUS, API_KEY_VALIDATION_FAILED);
		}
		
		$db = & JFactory::getDBO();
		$q = "SELECT param_value FROM #__remarkety WHERE param_name='REMARKETY_API_KEY'";
		$db->setQuery($q);
        $api_key = $db->loadResult();
		
		if($api_key == $request_api_key)
			return true;
		else{
			$this->returnJson(null, REMARKETY_FAILED_STATUS, API_KEY_VALIDATION_FAILED);
		}
	}
	
	/**
	 * 
	 * @return RemarketyModelRemarkety
	 */
	private function getRemarketyModel(){
		if($this->remarketyModel == null)
			$this->remarketyModel = $this->getModel();
		return $this->remarketyModel;
	}
	
	function loadVirtueMartFiles(){
		if(!$this->isInitialzedVM){
			global $mainframe, $auth, $db;
			$my = $mainframe->getUser();
			require_once ( JPATH_ADMINISTRATOR.DS."components".DS."com_virtuemart".DS."compat.joomla1.5.php");
			
			require_once ( JPATH_ADMINISTRATOR.DS."components".DS."com_virtuemart".DS."virtuemart.cfg.php");
			require_once( ADMINPATH. 'global.php' );
			require_once(CLASSPATH."ps_main.php");

			/*if (!class_exists('VmConfig')) require(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'helpers'.DS.'config.php');
			if (!class_exists('ShopFunctions')) require(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'helpers'.DS.'shopfunctions.php');
			if (!class_exists('VirtueMartModelCustomfields')) require(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'models'.DS.'customfields.php');
			if (!class_exists('CurrencyDisplay')) require(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_virtuemart'.DS.'helpers'.DS.'currencydisplay.php');
						
			VmConfig::loadConfig();
	
			$lang = JFactory::getLanguage();
			$lang->load('com_virtuemart', JPATH_BASE, null, false, false)
				||	$lang->load('com_virtuemart', JPATH_COMPONENT, null, false, false)
				||	$lang->load('com_virtuemart', JPATH_BASE, $lang->getDefault(), false, false)
				||	$lang->load('com_virtuemart', JPATH_COMPONENT, $lang->getDefault(), false, false);
				
			if(VmConfig::get('enableEnglish', 1)){
			    $lang->load('com_virtuemart', JPATH_ADMINISTRATOR, 'en-GB', true);
			    $lang->load('com_virtuemart', JPATH_ADMINISTRATOR, $lang->getDefault(), true);
			    $lang->load('com_virtuemart', JPATH_ADMINISTRATOR, null, true);
			}
			$lang->load('com_virtuemart', JPATH_SITE, $lang->getDefault(), true);
			$this->isInitialzedVM = true;*/
		}
	}
	
	static function getParamByName($name, $default = ''){
		$compParams = JComponentHelper::getParams('com_remarkety');
		$value = $compParams->get($name);
		if(empty($value)){
			return $default;
		}
		return $value;
	}

}
?>